// ----------------------------------------------------------------------------
//
// rm_folder version 2.0
//
// http://www.gilgil.net
//
// Copyright (c) Gilbert Lee All rights reserved
//
// ----------------------------------------------------------------------------

#include <boost/algorithm/string.hpp>
#include <boost/filesystem.hpp>
#include <iostream>
#include <vector>

using namespace std; // for string, cout, cin, vector
namespace fs = boost::filesystem;

#ifdef WIN32
#define DELIMITER '\\'
#endif
#ifdef linux
#define DELIMITER '/'
#endif

void usage()
{
  cout << "remove folder version 2.0\n";
  cout << "http://www.gilgil.net\n";
  cout << "Copyright (c) Gilbert Lee All rights reserved\n";
  cout << "\n";
  cout << "usage  : rm_folder [/q] [/i] [<root folder>] <folder name>\n";
  cout << "\n";
  cout << "         /q          : quiet mode\n";
  cout << "         /i          : case-insensitive comparison\n";
  cout << "         root folder : root folder(default : .)\n";
  cout << "         folder name : folder name to remove\n";
  cout << "\n";
  cout << "exam   : rm_folder .svn\n";
  cout << "         rm_folder /i debug\n";
  cout << "\n";
  cout << "caution: this program removes folders recursively.";
  cout << "\n";
}

// ----------------------------------------------------------------------------
// RemoveFolder
// ----------------------------------------------------------------------------
class RemoveFolder
{
public:
  bool   quiet;
  bool   insensitive;
  string folderName;
public:
  RemoveFolder();
  virtual ~RemoveFolder();
public:
  void scanFolder(string folder);
public:
  static string toupper(string s);
};

RemoveFolder::RemoveFolder()
{
  quiet       = false;
  insensitive = false;
  folderName  = "";
}

RemoveFolder::~RemoveFolder()
{
}

void RemoveFolder::scanFolder(string folder)
{
  fs::path p(folder);
  if (!fs::is_directory(p))
  {
    cout << p << " is not a directory\n";
    return;
  }
  typedef vector<fs::path> vec; vec v;
  copy(fs::directory_iterator(p), fs::directory_iterator(), back_inserter(v));
  for (vec::const_iterator it (v.begin()); it != v.end(); ++it)
  {
    //
    // Is it folder?
    //
    fs::path sub_p = *it;
    if (!fs::is_directory(sub_p)) continue;

    //
    // Extract sub folder name
    //
    string subFolderName = sub_p.filename().string();
    string subFolder     = folder + subFolderName + DELIMITER;

    //
    // if sub folder name = folder name?
    //
    bool isFolderNameSame = insensitive
      ? ( toupper(subFolderName) == toupper(folderName) )
      : ( subFolderName          == folderName );
    if (isFolderNameSame)
    {
      //
      // Remove sub folder.
      //
      bool ok = true;
      if (!quiet)
      {
        cout << "remove " << subFolder << "? ";
        char ch; cin >> ch;
        if (ch != 'Y' && ch != 'y') ok = false;
      }
      if (ok)
      {
        boost::uintmax_t res = fs::remove_all(subFolder);
        cout << "remove " << subFolder << " return " << res << endl;
      }
    } else
    {
      //
      // Scan sub folder recursively.
      //
      scanFolder(subFolder);
    }
  }
}

string RemoveFolder::toupper(string s)
{
  return boost::to_upper_copy(s);
}

int _main(int argc, char* argv[])
{
  RemoveFolder rf;
  string rootFolder = string(".") + DELIMITER;
  
  int index = 0;
  for (int i = 1; i < argc; i++)
  {
    string s = RemoveFolder::toupper(argv[i]);
    if (s == "/Q")
    {
      rf.quiet = true;
      continue;
    }
    if (s == "/I")
    {
      rf.insensitive = true;
      continue;
    }
    switch (index)
    {
      case 0 :
        rf.folderName = argv[i];
        index++;
        break;
      case 1 :
        rootFolder = rf.folderName;
        rf.folderName = argv[i];
        index++;
        break;
      default:
        cout << "invalid parameter\n";
        return 0;
    }
  }

  if (rf.folderName == "")
  {
    usage();
    return 0;
  }

  if (rootFolder.at(rootFolder.length() - 1) != DELIMITER)
  {
    rootFolder += DELIMITER;
  }

  rf.scanFolder(rootFolder);

  return 0;
}

int main(int argc, char* argv[])
{
  try {
    _main(argc, argv);
  }
  catch (fs::filesystem_error& e)
  {
    cout << e.what() << endl;
  }
  catch (...)
  {
    cout << "Unknown error occurred\n";
  }
}